from __future__ import division
from kivy.uix.widget import Widget
from kivy.core.window import Window
from util import import_it
from actor import Actor

class Scene(Actor):
    def __init__(self,game,**args):
        self.design_dimension = (1080,1920)
        self.game = game
        super(Scene,self).__init__(None,**args)
        self.init_configs()
        self.load()

    def init_properties(self,**args):
        super(Scene,self).init_properties(**args)
        self.fill_mode = 'stretch'
        ratio_x = Window.size[0] / self.design_dimension[0]
        ratio_y = Window.size[1] / self.design_dimension[1]
        if 'fill_mode' in args:
            self.fill_mode = args['fill_mode']
        self.size = Window.size 
        self.pos = (0,0)
        if self.fill_mode == 'fit_width':
            print 'fitting width'
            self.size = (Window.size[0],self.design_dimension[1] * ratio_x)
            self.pos = (0,(Window.size[1] - self.size[1]) / 2)
        elif self.fill_mode == 'fit_height':
            print 'fitting height'
            self.size= (self.design_dimension[0] * ratio_y,Window.size[1])
            self.pos = ((Window.size[0] - self.size[0]) / 2,0)
        self.x,self.y = self.pos
        self.width,self.height = self.size
        print self.size


    def update_size_pos(self):
        pass

    def update_dimension(self,actor):
        
        ratio_x = self.size[0] / self.design_dimension[0]
        ratio_y = self.size[1] / self.design_dimension[1]
        print Window.size
        print self.size
        print self.design_dimension
        print (ratio_x,ratio_y)
        actor.pos = (self.x+actor.design_pos[0] * ratio_x, self.y+actor.design_pos[1] * ratio_y)
        actor.size= (actor.design_size[0] * ratio_x, actor.design_size[1] * ratio_y)
        actor.x,actor.y = actor.pos
        actor.width,actor.height= actor.size
        
    
    def init_configs(self):
        self.handle_touch = False
        self.actor_configs = []
        self.triggers = {} 
        self.actors = []
        self.atlas = None
    
    def load(self):
        self.load_actors()
        self.load_triggers()
    
    def load_actors(self):
        for actor_config in self.actor_configs:
            self.load_actor(actor_config)

    
    def load_actor(self,actor_config):
        actor_class = Actor
        if 'class' in actor_config and len(actor_config['class']) > 0:
            actor_class = import_it(actor_config['class'])
        thing = actor_class(self,**actor_config)
        self.add_widget(thing)


    def trigger_event(self,event):
        pass






from kivy.app import App
from kivy.uix.widget import Widget
from kivy.core.window import Window
from kivy.clock import Clock
import ast
from kivy.atlas import Atlas
from actor import Actor
from scene import Scene
from util import import_it



class GameApp(App):
    current_scene = None
    game = None

    def __init__(self,game_file):
        super(GameApp,self).__init__()
        print 'starting game: %s'%(game_file)
        self.game = import_it(game_file) 
    def build(self):
        config = self.game.scenes[0]
        self.current_scene = import_it(config['class'])(self,**config)
        return self.current_scene




def import_it(name):
        mods = name.split('.')
        clz = None
        try:
            result = __import__(name)
        except:
            thing = '.'.join(mods[:-1])
            result = __import__(thing)
            clz = mods[-1]
            mods = mods[:-1]
        for mod in mods[1:]:
            result= getattr(result,mod)
        if clz is not None:
            result = getattr(result,clz)
        return result



from kivy.uix.widget import Widget
from kivy.graphics.context_instructions import Color

class Actor(Widget):

    def __init__(self,scene,**args):
        super(Actor,self).__init__(**args)
        self.scene= scene
        self.child_map = {}
        self.handle_touch = False
        self.touched = False
        self.design_size = (0,0)
        self.design_pos= (0,0)
        self.init_properties(**args)
        self.update_size_pos()

    def init_properties(self,**args):
        if 'design_size' in args:
            self.design_size = args['design_size']
        if 'design_pos' in args:
            self.design_pos= args['design_pos']
        if 'handle_touch' in args:
            self.handle_touch = args['handle_touch']
    
    
    def update_size_pos(self):
        if self.scene is not None:
            self.scene.update_dimension(self)

    def resize(self,size):
        self.design_size = size
        self.update_size_pos()

    def move(self,pos):
        self.design_pos = pos
        self.update_size_pos()

    
    def on_touch_down(self,touch):
        result = super(Actor,self).on_touch_down(touch)
        if self.handle_touch and not result and self.collide_point(touch.x,touch.y):
            self.touched = True
            return True 
        return False 

    def on_touch_move(self,touch):
        super(Actor,self).on_touch_move(touch)

    def on_touch_up(self,touch):
        result = super(Actor,self).on_touch_up(touch)
        if not result and self.touched and self.collide_point(touch.x,touch.y):
            self.on_click()
        self.touched = False
            
    def on_click(self):
        self.scene.trigger_event(('on_click',self.id))
    
    def add_widget(self,widget):
        super(Actor,self).add_widget(widget)
        if widget.id is not None:
            self.child_map[widget.id] = widget
        




from actor import Actor
from kivy.clock import Clock
from kivy.graphics.vertex_instructions import Rectangle
from kivy.atlas import Atlas

class ImageActor(Actor):
    def __init__(self,scene,**args):
        super(ImageActor,self).__init__(scene,**args)

    def init_properties(self,**args):
        super(ImageActor,self).init_properties(**args)
        self.states = {}
        self.rect = None
        self.frame = 0
        self.clock = None
        if 'states' in args:
            self.states = args['states']
        if self.states is not None and 'init' in self.states:
            self.switch('init')

    def get_texture(self):
        result = None
        if self.scene.atlas is not None and len(self.states)>0 and self.state in self.states:
            thing = self.states[self.state]
            if isinstance(thing,str):
                if thing in self.scene.atlas.textures:
                    result= self.scene.atlas.textures[thing]
            elif isinstance(thing,dict):
                key = thing['frames']
                delay = thing['delay']
                if key[self.frame] in self.scene.atlas.textures:
                    result= self.scene.atlas.textures[key[self.frame]]
                self.frame += 1
                if self.frame >= len(key):
                    self.frame = 0
                self.clock = Clock.schedule_once(self.apply_state,delay)

        return result

    def switch(self,state):
        Clock.unschedule(self.apply_state)
        self.state = state
        self.frame = 0
        self.apply_state()
        
    def apply_state(self,dt=0):
        self.clock = None
        if self.rect is None:
            self.rect = Rectangle(texture=self.get_texture(),size=self.size,pos=self.pos)
            self.canvas.add(self.rect)
        else:
            self.rect.size = self.size
            self.rect.pos = self.pos
            self.rect.texture = self.get_texture()




from distutils.core import setup
setup(
        name='kgame',
        packages=['kgame'],
        version = '0.93',
        description = 'a kivy game engine',
        author = 'Zhao Chang',
        author_email='zhchang@gmail.com',
        url='https://github.com/zhchang/kgame',
        keywords = ['kivy','game engine'],
        classifiers = [])
